package errors

import (
	"fmt"
	"github.com/gogf/gf/errors/gcode"
	"github.com/gogf/gf/errors/gerror"
	"sync"
)

var (
	codeNil      			 	= BizCode{0, "", ""}
	CodeSuccess                 = BizCode{1, "success", ""}                           // It is success.
	CodeInternalError     	 	= BizCode{50, "Internal Error", ""}
	CodeUnknown     	 		= BizCode{500, "An internal server error occurred", ""}
)

type BizCode struct {
	// C refers to the integer code of the ErrCode.
	C    int

	// M (user) facing error text.
	M string

	// Ref specify the reference document.
	Ref 	string
}

func init() {
	codes[CodeUnknown.Code()] = CodeUnknown
}

func (c BizCode) Code() int {
	return c.C
}

func (c BizCode) Message() string {
	return c.M
}

func (c BizCode) Detail() interface{} {
	return c.Ref
}

var codes = map[int]gcode.Code{}
var codeMux = &sync.Mutex{}

func MustRegister(coder gcode.Code) {
	if coder.Code()  < 1000  {
		panic("Codes less than 1000 are reserved as internal codes")
	}
	codeMux.Lock()
	defer codeMux.Unlock()

	if _, ok := codes[coder.Code()]; ok {
		panic(fmt.Sprintf("code: %d already exist", coder.Code()))
	}

	codes[coder.Code()] = coder
}

func ParseCoder(err error) gcode.Code {
	code := gerror.Code(err)
	if code == gcode.CodeNil && err != nil {
		code = CodeInternalError
	}

	if v, ok := code.(BizCode); ok {
		fmt.Println(333)
		if coder, ok := codes[v.C]; ok {
			return coder
		}
	}

	return CodeUnknown
}

func New( code int, message string, reference string) gcode.Code {
	return BizCode{
		C:    code,
		M: message,
		Ref : reference,
	}
}